@@ -33,34 +33,40 @@ import tables
 from ._utils import LOG, is_progress_disabled
 
 from .hierarchy import (
-    get_root_leaf_hog_offsets,
-    get_lca_hog_off,
-    get_descendant_species_taxoffs,
+    get_lca_off,
+    get_hog_child_prots, 
+    get_children,
+    get_hog2taxa
 )
+
 from .index import Index
 
 
 class Database(object):
-    # define the format of the tables
+
     class ProteinTableFormat(tables.IsDescription):
         ID = tables.StringCol(255, pos=1, dflt=b"")
-        SeqOff = tables.UInt64Col(pos=2)
-        SeqLen = tables.UInt64Col(pos=3)
-        SpeOff = tables.UInt64Col(pos=4)
-        HOGoff = tables.UInt64Col(pos=5)
-        FamOff = tables.UInt64Col(pos=6)
+        SpeOff = tables.UInt64Col(pos=2)
+        HOGoff = tables.UInt64Col(pos=3)
+        FamOff = tables.UInt64Col(pos=4)
+        SeqOff = tables.UInt64Col(pos=5)
+        SeqLen = tables.UInt64Col(pos=6)
 
     class HOGtableFormat(tables.IsDescription):
         ID = tables.StringCol(255, pos=1, dflt=b"")
-        FamOff = tables.UInt64Col(pos=2)
-        TaxOff = tables.UInt64Col(pos=3)
-        ParentOff = tables.Int64Col(pos=4)
-        ChildrenHOGoff = tables.Int64Col(pos=5)
-        ChildrenHOGnum = tables.Int64Col(pos=6)
-        ChildrenProtOff = tables.Int64Col(pos=7)
-        ChildrenProtNum = tables.Int64Col(pos=8)
-        OmaID = tables.StringCol(255, pos=9, dflt=b"")
-        LCAtaxOff = tables.UInt64Col(pos=10)
+        OmaID = tables.StringCol(255, pos=2, dflt=b"")
+        FamOff = tables.UInt64Col(pos=3)
+        TaxOff = tables.UInt64Col(pos=4)
+        ParentOff = tables.Int64Col(pos=5)
+        ChildrenOff = tables.Int64Col(pos=6)
+        ChildrenNum = tables.Int64Col(pos=7)
+        ChildrenProtOff = tables.Int64Col(pos=8)
+        ChildrenProtNum = tables.Int64Col(pos=9)
+        HOGtaxaOff = tables.Int64Col(pos=10)
+        HOGtaxaNum = tables.Int64Col(pos=11)
+        LCAtaxOff = tables.UInt64Col(pos=12)
+        NrMemberGenes = tables.Int64Col(pos=13)
+        CompletenessScore = tables.Float64Col(pos=14)
 
     class FamilyTableFormat(tables.IsDescription):
         ID = tables.UInt64Col(pos=1)
@@ -78,11 +84,12 @@ class Database(object):
 
     class TaxonomyTableFormat(tables.IsDescription):
         ID = tables.StringCol(255, pos=1, dflt=b"")
-        ParentOff = tables.Int64Col(pos=2)
-        ChildrenOff = tables.Int64Col(pos=3)
-        ChildrenNum = tables.Int64Col(pos=4)
-        SpeOff = tables.Int64Col(pos=5)
-        Level = tables.Int64Col(pos=6)
+        TaxID = tables.Int64Col(pos=2)
+        ParentOff = tables.Int64Col(pos=3)
+        ChildrenOff = tables.Int64Col(pos=4)
+        ChildrenNum = tables.Int64Col(pos=5)
+        SpeOff = tables.Int64Col(pos=6)
+        Level = tables.Int64Col(pos=7)
 
     def __init__(self, filename, root_taxon=None, mode='r', nthreads=1):
         assert (mode != 'w' or root_taxon), "A root_taxon must be defined when building the database"
@@ -189,6 +196,13 @@ class Database(object):
         else:
             return None
 
+    @property
+    def _hog_taxa_buff(self):
+        if "/HOGtaxa" in self.db:
+            return self.db.root.HOGtaxa
+        else:
+            return None
+
     @property
     def _ctax_arr(self):
         if "/ChildrenTax" in self.db:
@@ -259,6 +273,7 @@ class Database(object):
             tax_rows.append(
                 (
                     tax,
+                    -1,
                     par_off,
                     children_buffer_off if child_offsets else -1,
                     len(child_offsets),
@@ -286,7 +301,7 @@ class Database(object):
         return tax2taxoff, species
 
     def update_hog_and_fam_tabs(
-        self, fam2hogs, hog2taxoff, hog2protoffs, hog2oma_hog=None
+        self, fam2hogs, hog2taxoff, hog2protoffs, hog2oma_hog=None, hog2gene_nr=None, hog2completeness=None
     ):
         assert self.mode in {"w", "a"}, "Database must be opened in write mode."
 
@@ -438,10 +453,24 @@ class Database(object):
                 else repeat(b"", hog_num)
             )
 
+            # quality measures
+            oma_hog_sizes = (
+                list(map(lambda x: hog2gene_nr[x], hogs))
+                if hog2gene_nr
+                else repeat(-1, hog_num)
+            )
+            
+            oma_hog_comps = (
+                list(map(lambda x: hog2completeness[x], hogs))
+                if hog2completeness
+                else repeat(-1, hog_num)
+            )
+
             hog_rows.extend(
                 list(
                     zip(
                         hogs,
+                        oma_hogs,
                         repeat(fam_off),
                         hog_taxoffs,
                         parents,
@@ -449,8 +478,11 @@ class Database(object):
                         child_hogs_numbers,
                         child_prots_offsets,
                         child_prots_numbers,
-                        oma_hogs,
+                        repeat(-1),
+                        repeat(-1),
                         repeat(0),
+                        oma_hog_sizes,
+                        oma_hog_comps
                     )
                 )
             )
@@ -463,7 +495,7 @@ class Database(object):
                     hog_off,
                     hog_num,
                     level_offsets_off,
-                    hog_level_offsets_num,
+                    hog_level_offsets_num
                 )
             )
 
@@ -565,26 +597,58 @@ class Database(object):
         self._prot_tab.modify_column(colname="HOGoff", column=hogoff_col)
         self._prot_tab.modify_column(colname="FamOff", column=famoff_col)
 
-    def add_lcataxoff_col(self):
-        """
-        compute the LCA taxon between the HOG species members
-        """
-        hog_tab = self._hog_tab[:]
-        chog_buff = self._chog_arr[:]
-        cprot_buff = self._cprot_arr[:]
-        prot2speoff = self._prot_tab.col("SpeOff")
-        speoff2taxoff = self._sp_tab.col("TaxOff")
-        parent_arr = self._tax_tab.col("ParentOff")
+    def store_hog2taxa(self):
+        '''
+        Store taxonomic levels of HOGs.
+        '''
+        hog_taxa_idx, hog_taxa_buff = get_hog2taxa(
+            self._hog_tab[:], self._sp_tab[:], self._prot_tab[:], self._cprot_arr[:], self._tax_tab[:], self._chog_arr[:])
 
-        lcatax_offsets = np.zeros(hog_tab.size, dtype=np.uint64)
+        self.db.create_carray(
+                        "/",
+                        "HOGtaxa",
+                        obj=hog_taxa_buff,
+                        filters=self._compr)
+
+        self._hog_tab.modify_column(colname='HOGtaxaOff', column=hog_taxa_idx[:-1])
+        self._hog_tab.modify_column(colname='HOGtaxaNum', column=hog_taxa_idx[1:] - hog_taxa_idx[:-1])
+
+    def add_lcataxoff_col(self):
+        '''
+        Compute LCA taxon among HOG members.
+        '''
+        hog_tab = self._hog_tab[:]
+        prot_tab = self._prot_tab[:]
+        cprot_buff = self._cprot_arr[:]
+        chog_buff = self._chog_arr[:]
+        tax_tab = self._tax_tab[:]
+        sp_tab = self._sp_tab[:]
+
+        lca_tax_offsets = np.zeros(hog_tab.size, dtype=np.uint64)
 
         for hog_off in range(hog_tab.size):
-            species_tax_offs = get_descendant_species_taxoffs(
-                hog_off, hog_tab, chog_buff, cprot_buff, prot2speoff, speoff2taxoff
-            )
-            lcatax_offsets[hog_off] = get_lca_hog_off(species_tax_offs, parent_arr)
+            hog_ms_taxa = np.append(np.unique(sp_tab['TaxOff'][prot_tab['SpeOff'][get_hog_child_prots(hog_off, hog_tab, cprot_buff)]]), 
+                                    np.unique(hog_tab['TaxOff'][get_children(hog_off, hog_tab, chog_buff)]))
+            lca_tax_offsets[hog_off] = get_lca_off(hog_ms_taxa, tax_tab['ParentOff'])
+            
+        self._hog_tab.modify_column(colname="LCAtaxOff", column=lca_tax_offsets)
 
-        self._hog_tab.modify_column(colname="LCAtaxOff", column=lcatax_offsets)
+        # hog_tab = self._hog_tab[:]
+        # chog_buff = self._chog_arr[:]
+        # cprot_buff = self._cprot_arr[:]
+        # prot2speoff = self._prot_tab.col("SpeOff")
+        # speoff2taxoff = self._sp_tab.col("TaxOff")
+        # parent_arr = self._tax_tab.col("ParentOff")
+
+        # lcatax_offsets = np.zeros(hog_tab.size, dtype=np.uint64)
+
+        # for hog_off in range(hog_tab.size):
+        #     species_tax_offs = get_descendant_species_taxoffs(
+        #         hog_off, hog_tab, chog_buff, cprot_buff, prot2speoff, speoff2taxoff
+        #     )
+        #     lcatax_offsets[hog_off] = get_lca_off(species_tax_offs, parent_arr)
+
+        # self._hog_tab.modify_column(colname="LCAtaxOff", column=lcatax_offsets)
 
     ### generic functions ###
     @staticmethod
@@ -616,10 +680,12 @@ class DatabaseFromOMA(Database):
     """
     Used to parse the OMA browser database file
     """
-    def __init__(self, filename, root_taxon, min_prot_nr=6, include_younger_fams=True, mode='r'):
+    def __init__(self, filename, root_taxon, min_fam_size=6, min_fam_completeness=0, logic='AND', include_younger_fams=True, mode='r'):
         super().__init__(filename, root_taxon, mode=mode)
 
-        self.min_prot_nr = min_prot_nr
+        self.min_fam_size = min_fam_size
+        self.min_fam_completeness = min_fam_completeness
+        self.logic = logic
         self.include_younger_fams = include_younger_fams
 
     ### main function ###
@@ -634,7 +700,7 @@ class DatabaseFromOMA(Database):
         h5file = tables.open_file(oma_h5_path, mode="r")
 
         LOG.debug("select and strip OMA HOGs")
-        fam2hogs, hog2oma_hog, hog2tax = self.select_and_strip_OMA_HOGs(h5file)
+        fam2hogs, hog2oma_hog, hog2tax, hog2gene_nr, hog2completeness = self.select_and_strip_OMA_HOGs(h5file)
 
         LOG.debug("fill sequence buffer, species table and initiate protein table")
         (
@@ -648,7 +714,7 @@ class DatabaseFromOMA(Database):
             hog2oma_hog,
             hog2tax,
             species,
-            self.min_prot_nr
+            self.min_fam_size
         )
 
         LOG.debug(
@@ -661,12 +727,15 @@ class DatabaseFromOMA(Database):
         hog2taxoff = {h: tax2taxoff.get(t, -1) for h, t in hog2tax.items()}
 
         LOG.debug("fill family and HOG tables")
-        self.update_hog_and_fam_tabs(fam2hogs, hog2taxoff, hog2protoffs, hog2oma_hog)
+        self.update_hog_and_fam_tabs(fam2hogs, hog2taxoff, hog2protoffs, hog2oma_hog, hog2gene_nr, hog2completeness)
 
         # add family and hog offsets
         LOG.debug("complete protein table")
         self.update_prot_tab(hog2protoffs)
 
+        LOG.debug("store HOG taxa")
+        self.store_hog2taxa()
+
         LOG.debug("compute LCA taxa")
         self.add_lcataxoff_col()
 
@@ -675,36 +744,45 @@ class DatabaseFromOMA(Database):
 
     ### functions to parse OMA database ###
     def select_and_strip_OMA_HOGs(self, h5file):
+        
         def _process_oma_hog(
             tax2level,
             curr_oma_taxa,
             curr_oma_hog,
             curr_oma_roothog,
+            curr_oma_roothog_ok,
             fam,
             fam2hogs,
             hog2oma_hog,
             hog2tax,
+            hog2gene_nr,
+            hog2completeness,
             roottax,
-            include_younger_fams
+            include_younger_fams,
+            curr_oma_sizes,
+            min_fam_size,
+            curr_oma_comps,
+            min_fam_completeness,
+            logic
         ):
             """
-			- decide whether an OMA HOG should be stored based on current root-HOG and HOG taxa
-			- update root-HOG if necessary and keep track of HOG taxa
-			"""
+            - decide whether an OMA HOG should be stored based on current root-HOG and HOG taxa
+            - update root-HOG if necessary and keep track of HOG taxa
+            """
 
             def _is_descendant(hog1, hog2):
                 """
-			    True if hog1 descendant of hog2
-			    """
+                True if hog1 descendant of hog2
+                """
                 return hog2 in self.parse_hogs(hog1)
 
             def _store(
-                fam, curr_oma_hog, curr_oma_roothog, fam2hogs, hog2oma_hog, hog2tax, tax
-            ):
+                fam, curr_oma_hog, curr_oma_roothog, fam2hogs, hog2oma_hog, hog2tax, hog2gene_nr, hog2completeness,
+                tax, hog_size, hog_comp):
+                """
+                - compute SPhog HOG id
+                - fill containers
                 """
-			    - compute SPhog HOG id
-			    - fill containers
-			    """
                 hog = b".".join(
                     [
                         str(fam).encode("ascii"),
@@ -715,144 +793,150 @@ class DatabaseFromOMA(Database):
                 fam2hogs[fam].add(hog)
                 hog2oma_hog[hog] = curr_oma_hog
                 hog2tax[hog] = tax
+                hog2gene_nr[hog] = hog_size
+                hog2completeness[hog] = hog_comp
 
             # compute most ancestral taxon; when absent, flag it with 1000000
             tax_levels = list(map(lambda x: tax2level.get(x, 1000000), curr_oma_taxa))
             min_level = min(tax_levels)
-            
-            # skip HOGs outside of taxonomic scope defined by the root-taxon
+
+            # if none of the HOG taxa within the taxonomic scope defined by the root-taxon, skip the HOG
             if min_level == 1000000:
-                return fam, curr_oma_roothog
-            else:
-                tax = curr_oma_taxa[tax_levels.index(min_level)]  
-                
-                # whether HOG is descendant of current OMA root-HOG 
-                if _is_descendant(curr_oma_hog, curr_oma_roothog):
-
-                    _store(
-                        fam,
-                        curr_oma_hog,
-                        curr_oma_roothog,
-                        fam2hogs,
-                        hog2oma_hog,
-                        hog2tax,
-                        tax,
-                    )
-                
-                # or create a new family (include only family at root-taxon if include_younger_fams==False)
-                elif (not include_younger_fams and tax == roottax) or include_younger_fams:
-
-                    fam += 1
-                    curr_oma_roothog = curr_oma_hog
-
-                    # store after updating fam
-                    _store(
-                        fam,
-                        curr_oma_hog,
-                        curr_oma_roothog,
-                        fam2hogs,
-                        hog2oma_hog,
-                        hog2tax,
-                        tax,
-                    )
-
-            return fam, curr_oma_roothog
+                pass
             
-            # # compute most ancestral taxon; when absent, flag it with 1000000
-            # tax_levels = list(map(lambda x: tax2level.get(x, 1000000), curr_oma_taxa))
-            # min_level = min(tax_levels)
+            else:
+                # the HOG taxon is the older taxon within the taxonimic scope defined by the root-taxon
+                idx = tax_levels.index(min_level)
+                hog_tax = curr_oma_taxa[idx]  
+                hog_size = curr_oma_sizes[idx]
+                hog_comp = curr_oma_comps[idx]
+                
+                # if the HOG is descendant of current OMA root-HOG, new sub-HOG
+                if _is_descendant(curr_oma_hog, curr_oma_roothog):
+                    
+                    # but store only if root-HOG passed quality thresholds
+                    if curr_oma_roothog_ok:
+                        
+                        _store(
+                            fam,
+                            curr_oma_hog,
+                            curr_oma_roothog,
+                            fam2hogs,
+                            hog2oma_hog,
+                            hog2tax,
+                            hog2gene_nr,
+                            hog2completeness,
+                            hog_tax,
+                            hog_size,
+                            hog_comp
+                        )
 
-            # if min_level == 1000000:
-            #     tax = None
-            # else:
-            #     tax = curr_oma_taxa[tax_levels.index(min_level)]
+                # else, new root-HOG (include only root-HOG at root-taxon if include_younger_fams==False)
+                elif (not include_younger_fams and hog_tax == roottax) or include_younger_fams:
+                    
+                    curr_oma_roothog = curr_oma_hog
+                        
+                    # but store it only if passes quality thresholds
+                    ok = (hog_size >= min_fam_size or hog_comp >= min_fam_completeness) if logic == 'OR' else (hog_size >= min_fam_size and hog_comp >= min_fam_completeness)
+                    if ok:
+                        
+                        fam += 1
+                        curr_oma_roothog_ok = True
+                        
+                        # store after updating fam
+                        _store(
+                            fam,
+                            curr_oma_hog,
+                            curr_oma_roothog,
+                            fam2hogs,
+                            hog2oma_hog,
+                            hog2tax,
+                            hog2gene_nr,
+                            hog2completeness,
+                            hog_tax,
+                            hog_size,
+                            hog_comp
+                        )
+                        
+                    else:
+                        curr_oma_roothog_ok = False
 
-            # # store if descendant of current OMA root-HOG
-            # if _is_descendant(curr_oma_hog, curr_oma_roothog):
-
-            #     _store(
-            #         fam,
-            #         curr_oma_hog,
-            #         curr_oma_roothog,
-            #         fam2hogs,
-            #         hog2oma_hog,
-            #         hog2tax,
-            #         tax,
-            #     )
-
-            # # if tax equal or younger that root taxon: store, update fam and current OMA root-HOG
-            # elif tax:
-
-            #     # include only family at root-taxon if include_younger_fams==False
-            #     if (not include_younger_fams and tax == roottax) or include_younger_fams:
-
-            #         fam += 1
-            #         curr_oma_roothog = curr_oma_hog
-
-            #         # store after updating fam
-            #         _store(
-            #             fam,
-            #             curr_oma_hog,
-            #             curr_oma_roothog,
-            #             fam2hogs,
-            #             hog2oma_hog,
-            #             hog2tax,
-            #             tax,
-            #         )
-
-            # return fam, curr_oma_roothog
+            return fam, curr_oma_roothog, curr_oma_roothog_ok
 
         def _process_oma_fam(
-            fam_tab_sort, tax2level, fam, fam2hogs, hog2oma_hog, hog2tax, roottax, include_younger_fams
-        ):
+            fam_tab_sort, tax2level, fam, fam2hogs, hog2oma_hog, hog2tax, hog2gene_nr, hog2completeness, 
+            roottax, include_younger_fams, min_fam_size, min_fam_completeness, logic):
+            """
+            apply _process_oma_hog to one OMA family
+             - fam_tab_sort: slice of the HogLevel table for one family sorted by HOG ids
+             - tax2level: mapper between taxa and their distance from roottax
             """
-			apply _process_oma_hog to one OMA family
-			 - fam_tab_sort: slice of the HogLevel table for one family sorted by HOG ids
-			 - tax2level: mapper between taxa and their distance from roottax
-			"""
             # bookeepers for HOGs
             curr_oma_hog = fam_tab_sort[0]["ID"]
             curr_oma_roothog = None
+            curr_oma_roothog_ok = False
             curr_oma_taxa = []
+            curr_oma_comps = []
+            curr_oma_sizes = []
 
             for r in fam_tab_sort:
-                oma_hog, oma_tax = r[1], r[2] #b"".join(r[2].split())
+                oma_hog, oma_tax, hog_completeness, hog_gene_nr = r[1], r[2], r[3], r[5]
 
                 # evaluation at new oma HOG
                 if oma_hog != curr_oma_hog:
 
-                    fam, curr_oma_roothog = _process_oma_hog(
+                    fam, curr_oma_roothog, curr_oma_roothog_ok = _process_oma_hog(
                         tax2level,
                         curr_oma_taxa,
                         curr_oma_hog,
                         curr_oma_roothog,
+                        curr_oma_roothog_ok,
                         fam,
                         fam2hogs,
                         hog2oma_hog,
                         hog2tax,
+                        hog2gene_nr,
+                        hog2completeness,
                         roottax,
-                        include_younger_fams
+                        include_younger_fams,
+                        curr_oma_sizes,
+                        min_fam_size,
+                        curr_oma_comps,
+                        min_fam_completeness,
+                        logic
                     )
 
                     # reset for new HOG
                     curr_oma_taxa = []
+                    curr_oma_sizes = []
+                    curr_oma_comps = []
                     curr_oma_hog = oma_hog
 
-                # track taxa of current oma HOG
+                # track taxa, completeness and size of current oma HOG
                 curr_oma_taxa.append(oma_tax)
+                curr_oma_comps.append(hog_completeness)
+                curr_oma_sizes.append(hog_gene_nr)
 
             # end
-            fam, curr_oma_roothog = _process_oma_hog(
+            fam, curr_oma_roothog, curr_oma_roothog_ok = _process_oma_hog(
                 tax2level,
                 curr_oma_taxa,
                 curr_oma_hog,
                 curr_oma_roothog,
+                curr_oma_roothog_ok,
                 fam,
                 fam2hogs,
                 hog2oma_hog,
                 hog2tax,
+                hog2gene_nr,
+                hog2completeness,
                 roottax,
-                include_younger_fams
+                include_younger_fams,
+                curr_oma_sizes,
+                min_fam_size,
+                curr_oma_comps,
+                min_fam_completeness,
+                logic
             )
 
             return fam
@@ -861,37 +945,45 @@ class DatabaseFromOMA(Database):
         tax2level = dict(zip(self._tax_tab[:]["ID"], self._tax_tab[:]["Level"]))
         hog_tab = h5file.root.HogLevel
 
-        # load families in memory
-        families = hog_tab.col("Fam")
-
         # containers
         fam2hogs = collections.defaultdict(set)
         hog2oma_hog = dict()
         hog2tax = dict()
+        hog2gene_nr = dict()
+        hog2completeness = dict()
 
         # bookeepers for families
         fam = 0
-        curr_fam = families[0]
+        curr_fam = hog_tab[0]['Fam']
         i = 0
         j = 0
 
-        for oma_fam in tqdm(families, disable=is_progress_disabled()):
+        for hog_ent in hog_tab:
+
+        # for hog_ent in tqdm(hog_tab, disable=is_progress_disabled()):
+            oma_fam = hog_ent['Fam']
+
             if oma_fam != curr_fam:
 
                 # load fam table and sort by HOG ids
                 fam_tab = hog_tab[i:j]
-                fam_tab = np.sort(fam_tab, order="ID")
+                fam_tab_sort = np.sort(fam_tab, order="ID")
 
                 # select and strip HOGs of one family
                 fam = _process_oma_fam(
-                    fam_tab,
+                    fam_tab_sort,
                     tax2level,
                     fam,
                     fam2hogs,
                     hog2oma_hog,
                     hog2tax,
+                    hog2gene_nr,
+                    hog2completeness,    
                     self.root_taxon.encode("ascii"),
-                    self.include_younger_fams
+                    self.include_younger_fams,
+                    self.min_fam_size,
+                    self.min_fam_completeness,
+                    self.logic
                 )
 
                 # move pointer and update current family
@@ -909,46 +1001,53 @@ class DatabaseFromOMA(Database):
             fam2hogs,
             hog2oma_hog,
             hog2tax,
+            hog2gene_nr,
+            hog2completeness,    
             self.root_taxon.encode("ascii"),
-            self.include_younger_fams
+            self.include_younger_fams,
+            self.min_fam_size,
+            self.min_fam_completeness,
+            self.logic
         )
 
-        del hog_tab, families
+        del hog_tab
 
-        return fam2hogs, hog2oma_hog, hog2tax
+        return fam2hogs, hog2oma_hog, hog2tax, hog2gene_nr, hog2completeness
 
     def select_and_filter_OMA_proteins(
-        self, h5file, fam2hogs, hog2oma_hog, hog2tax, species, min_prot_nr
+        self, h5file, fam2hogs, hog2oma_hog, hog2tax, species, min_fam_size
     ):
         """
         One small diff compared to DatabaseFromFasta is that proteins in protein table are not following the species in species table. This should be fine.
         """
         genome_tab = h5file.root.Genome[:]
-        ent_tab = h5file.root.Protein.Entries
         oma_seq_buffer = h5file.root.Protein.SequenceBuffer
+        ent_tab = h5file.root.Protein.Entries
 
-        # temporary mappers
+        # temporary mappers and booking for latter
+        sp2sp_off = dict(zip(sorted(species), range(len(species))))  # this is to sort the species table
         oma_hog2hog = dict(zip(hog2oma_hog.values(), hog2oma_hog.keys()))
-        spe2speoff = dict(
-            zip(sorted(species), range(len(species)))
-        )  # this is to sort the species table
+        hog2protoffs = collections.defaultdict(set)
 
         LOG.debug(" - select proteins from selected HOGs")
-        # remember the size of families and bookkeep family of each protein for filtering
-        fam2prot_nr = collections.Counter()
-        prot_fams = []
 
-        # bookkeep HOG of each protein to later build the hog2protoffs mapper
-        prot_hogs = []
+        # track current species
+        curr_sp = None
 
-        # bookkeep sequence offset of each protein to later build the local seq_buff
-        oma_seq_offsets = []
+        # pointer to sequence in buffer
+        seq_off = self._seq_buff.nrows
 
-        # store temporary rows for species and protein tables
-        oma_species = [b""] * len(species)  # to keep species sorted
+        # pointer to protein in protein table
+        prot_off = self._prot_tab.nrows
+        curr_prot_off = prot_off
+
+        # store rows for species and protein tables and sequence buffer
+        sp_rows = [()] * len(species)  # keep sorted
+        seq_buff = []
         prot_rows = []
 
-        for r in tqdm(genome_tab, disable=is_progress_disabled()):
+        for r in genome_tab:
+        # for r in tqdm(genome_tab, disable=is_progress_disabled()):
 
             sp = r['SciName']
             sp_code = r['UniProtSpeciesCode']
@@ -960,131 +1059,85 @@ class DatabaseFromOMA(Database):
 
             # filter if species outside root-taxon
             if sp in species:
-                entry_off = r["EntryOff"]
-                entry_num = r["TotEntries"]
+                sp_off = sp2sp_off[sp]  
 
-                spe_off = spe2speoff[sp]
-                oma_species[spe_off] = sp
-
-                # sub entry table for species
-                sp_ent_tab = ent_tab[entry_off : entry_off + entry_num]
-
-                for rr in sp_ent_tab:
-                    oma_hog = rr["OmaHOG"]
-                    # select protein if member of selected OMA HOG
-                    if oma_hog in oma_hog2hog:
-                        # update counter and track hogs and families
-                        hog = oma_hog2hog[oma_hog]
-                        fam = int(hog.split(b".")[0].decode("ascii"))
-                        fam2prot_nr[fam] += 1
-                        prot_hogs.append(hog)
-                        prot_fams.append(fam)
-
-                        # track sequence length and offset
-                        oma_seq_off = rr["SeqBufferOffset"]
-                        seq_len = rr["SeqBufferLength"]
-                        oma_seq_offsets.append(oma_seq_off)
-
-                        # store protein row
-                        oma_id = '{}{:05d}'.format(sp_code.decode('ascii'), rr['EntryNr'] - entry_off)
-                        prot_rows.append((oma_id.encode('ascii'), 0, seq_len, spe_off, 0, 0))
-
-        LOG.debug(" - filter by family protein number")
-        # now filter the mappers
-        f_fam2hogs = collections.defaultdict(set)
-        f_hog2oma_hog = dict()
-        f_hog2tax = dict()
-
-        for fam, hogs in fam2hogs.items():
-
-            prot_nr = fam2prot_nr[fam]
-
-            # filter by size
-            if prot_nr >= min_prot_nr:
-
-                f_fam2hogs[fam] = hogs
-                f_hog2oma_hog.update(
-                    dict(zip(hogs, list(map(lambda x: hog2oma_hog[x], hogs))))
-                )
-                f_hog2tax.update(dict(zip(hogs, list(map(lambda x: hog2tax[x], hogs)))))
-
-        LOG.debug(" - filter proteins based on filtered families")
-        # bookkeeping for later
-        hog2protoffs = collections.defaultdict(set)
-
-        # pointer to species in species table
-        curr_spe_off = prot_rows[0][3]
-
-        # pointer to protein in protein table
-        prot_off = self._prot_tab.nrows
-        curr_prot_off = prot_off
-
-        # pointer to sequence in buffer
-        seq_off = self._seq_buff.nrows
-
-        # store rows for species and protein tables	and sequence buffer
-        spe_rows = [()] * len(species)  # keep sorted
-        f_prot_rows = []
-        seq_buff = []
-
-        # iter trough temporary protein rows
-        i = 0
-        for r in tqdm(prot_rows, disable=is_progress_disabled()):
-            if prot_fams[i] in f_fam2hogs:
-                spe_off = r[3]
+                # initiate curr_spe
+                if not curr_sp: 
+                    curr_sp = sp
 
                 # change of species --> store current species
-                if spe_off != curr_spe_off:
-
-                    spe_rows[curr_spe_off] = (
-                        oma_species[curr_spe_off],
+                if sp != curr_sp:
+                         
+                    sp_rows[sp2sp_off[curr_sp]] = (
+                        curr_sp,
                         curr_prot_off,
                         prot_off - curr_prot_off,
                         0,
                     )
 
                     curr_prot_off = prot_off
-                    curr_spe_off = spe_off
+                    curr_sp = sp
 
                     # take advantage to dump sequence buffer
                     self._seq_buff.append(seq_buff)
                     self._seq_buff.flush()
                     seq_buff = []
 
-                # sequence
-                oma_seq_off = oma_seq_offsets[i]
-                seq_len = r[2]
-                seq = oma_seq_buffer[oma_seq_off : oma_seq_off + seq_len]
-                seq_buff.extend(list(seq))
+                # load sub entry table for species
+                entry_off = r["EntryOff"]
+                entry_num = r["TotEntries"]
+                sp_ent_tab = ent_tab[entry_off : entry_off + entry_num]
 
-                # store protein row
-                f_prot_rows.append((r[0], seq_off, seq_len, spe_off, 0, 0))
+                for rr in sp_ent_tab:
+                    oma_hog = rr["OmaHOG"]
+                    
+                    # select protein if member of selected OMA HOG
+                    if oma_hog not in oma_hog2hog:
+                        continue
 
-                # book keeping
-                hog2protoffs[prot_hogs[i]].add(prot_off)
+                    # sequence
+                    oma_seq_off = rr["SeqBufferOffset"]
+                    seq_len = rr["SeqBufferLength"]
+                    seq = oma_seq_buffer[oma_seq_off : oma_seq_off + seq_len]    
+                    seq_buff.extend(list(seq))      
+                    
+                    # store protein row
+                    oma_id = '{}{:05d}'.format(sp_code.decode('ascii'), rr['EntryNr'] - entry_off)
+                    prot_rows.append((oma_id.encode('ascii'), sp_off, 0, 0, seq_off, seq_len))                        
+                    
+                    # track hog and family
+                    hog = oma_hog2hog[oma_hog]
+                    fam = int(hog.split(b".")[0].decode("ascii"))
+                    hog2protoffs[hog].add(prot_off)
 
-                # update offset of protein sequence in buffer and of protein raw in table
-                seq_off += seq_len
-                prot_off += 1
-            i += 1
+                    # update offset of protein sequence in buffer and of protein raw in table
+                    seq_off += seq_len
+                    prot_off += 1     
 
         # end
-        spe_rows[curr_spe_off] = (
-            oma_species[curr_spe_off],
+        sp_rows[sp2sp_off[curr_sp]] = (
+            curr_sp,
             curr_prot_off,
             prot_off - curr_prot_off,
             0,
         )
-
         self._seq_buff.append(seq_buff)
         self._seq_buff.flush()
 
         # fill species and protein tables
-        self._sp_tab.append(spe_rows)
+        self._sp_tab.append(sp_rows)
         self._sp_tab.flush()
-        self._prot_tab.append(f_prot_rows)
+        self._prot_tab.append(prot_rows)
         self._prot_tab.flush()
 
         del genome_tab, ent_tab, oma_seq_buffer
 
-        return f_fam2hogs, hog2protoffs, f_hog2tax, f_hog2oma_hog
+        return fam2hogs, hog2protoffs, hog2tax, hog2oma_hog
+
+    def add_taxid_col(self, h5file):
+        '''
+        Parse NCBI taxonomic ids from OMA database
+        '''
+        oma_tax_tab = h5file.root.Taxonomy[:]
+        tax2taxid = dict(zip(oma_tax_tab['Name'], oma_tax_tab['NCBITaxonId']))
+        self._tax_tab.modify_column(colname="TaxID", column=np.array(list(map(lambda x: tax2taxid[x], self._tax_tab.col('ID'))), dtype=np.int64))
