@@ -35,7 +35,11 @@ import tables
 
 from ._utils import LOG
 from .alphabets import Alphabet
-from .hierarchy import get_lca_hog_off, get_descendant_species, get_descendant_taxa
+from .hierarchy import (
+    get_lca_off, 
+    get_descendants,
+    get_leaves
+)
 
 
 @numba.njit
@@ -69,25 +73,26 @@ class Index(object):
         # performance features
         self.nthreads = nthreads
 
-    ### useful for validation
-    def set_species(self, sp_ii):
-        self.sp_filter = np.full((len(self.db._sp_tab),), True)
-        #self.sp_filter[:] = True
-        for i in sp_ii:
-            self.sp_filter[i] = False
+    # ### useful for validation
+    # def set_species(self, sp_ii):
+    #     self.sp_filter = np.full((len(self.db._sp_tab),), True)
+    #     #self.sp_filter[:] = True
+    #     for i in sp_ii:
+    #         self.sp_filter[i] = False
 
     @property
     def sp_filter(self):
+        tax_tab = self.db._tax_tab[:]
         sp_filter = np.full((len(self.db._sp_tab),), False)
         for hidden_taxon in self.hidden_taxa:
-            descendant_species = get_descendant_species(
-                np.searchsorted(self.db._tax_tab.col('ID'), hidden_taxon.encode('ascii')), self.db._tax_tab, self.db._ctax_arr)
+            descendant_species = get_leaves(
+                np.searchsorted(tax_tab['ID'], hidden_taxon.encode('ascii')), tax_tab, self.db._ctax_arr[:])
             # case where hidden taxon is species
             if descendant_species.size == 0:
                 sp_filter[np.searchsorted(self.db._sp_tab.col('ID'), hidden_taxon.encode('ascii'))] = True
             else:
                 for sp_off in descendant_species:
-                    sp_filter[sp_off] = True
+                    sp_filter[tax_tab['SpeOff'][sp_off]] = True
         return sp_filter
 
     @property
@@ -96,7 +101,7 @@ class Index(object):
         for hidden_taxon in self.hidden_taxa:
             htax_off = np.searchsorted(self.db._tax_tab.col('ID'), hidden_taxon.encode('ascii'))
             tax_filter[htax_off] = True
-            descendant_taxa = get_descendant_taxa(htax_off, self.db._tax_tab, self.db._ctax_arr)
+            descendant_taxa = get_descendants(htax_off, self.db._tax_tab, self.db._ctax_arr)
             for htax_off in descendant_taxa:
                 tax_filter[htax_off] = True
         return tax_filter
@@ -125,7 +130,6 @@ class Index(object):
     ### main function to build the index ###
     def build_kmer_table(self):
 
-        #assert self.db.mode in {"w", "a"}, "Index must be opened in write mode."
         assert self.db.mode == 'w', "Index must be opened in write mode."
         assert ('/Index' not in self.db.db), 'Index has already been computed'
 
@@ -147,7 +151,6 @@ class Index(object):
         return sa
 
     def _build_kmer_table(self, sa):
-        #@numba.njit(parallel=True) --> was breaking for PANTHER database
         @numba.njit
         def _compute_mask_and_filter(
             sa, sa_mask, sa_filter, k, n, prot2spoff, prot2hogoff, sp_filter
@@ -192,13 +195,13 @@ class Index(object):
                 if fam == curr_fam:
                     curr_hogs.append(hog_offsets[i])
                 else:
-                    lca_hogs[lca_off] = get_lca_hog_off(curr_hogs, hog_parents)
+                    lca_hogs[lca_off] = get_lca_off(curr_hogs, hog_parents)
                     curr_hogs = list(hog_offsets[i : i + 1])
                     curr_fam = fam
                     lca_off += 1
 
             # last family
-            lca_hogs[lca_off] = get_lca_hog_off(curr_hogs, hog_parents)
+            lca_hogs[lca_off] = get_lca_off(curr_hogs, hog_parents)
             return lca_hogs
 
         @numba.njit
@@ -348,29 +351,24 @@ class Index(object):
             idx, "HOGcount", obj=hog_kmer_counts, filters=self.db._compr
         )
 
-
 class SequenceBuffer(object):
-    """
-    to load sequences from db or files
-    adapted from alex
-    """
-
-    def __init__(self, seqs=None, ids=None, db=None, fasta_file=None):
-        self.prot_off = 0
-        if seqs is not None:
-            self.add_seqs(*seqs)
-            self.ids = np.array(ids) if ids else np.array(range(len(seqs)))
-        elif db is not None:
-            self._prot_tab = db._prot_tab
-            self._seq_buff = db._seq_buff
-            self.load_from_db()
-        elif fasta_file is not None:
-            seqs = self.parse_fasta(fasta_file)
-            self.add_seqs(*seqs)
-        else:
-            raise ValueError(
-                "need to pass array of seqs or db to load from to SequenceBuffer"
-            )
+    '''
+    makes list of sequences search friendly
+    adapted from Alex
+    TO DO:
+    - sanitize seqs
+    - debug reduced alphabet
+    - (debug __getitem__, which does not work with negative values)
+    '''
+    def __init__(self, seqs, ids, alphabet_n=21):
+        '''
+        args:
+         - seqs: list of sequences
+         - ids: list of corresponding ids
+        '''
+        self.alphabet = Alphabet(n=alphabet_n)
+        self.add_seqs(*seqs)
+        self.ids = (np.array(ids) if ids else np.array(range(len(seqs))))
 
     def __getstate__():
         return (self.prot_nr, self.n, self.buff_shr, self.buff_idx_shr)
@@ -378,6 +376,38 @@ class SequenceBuffer(object):
     def __setstate__(state):
         (self.prot_nr, self.n, self.buff_shr, self.buff_idx_shr) = state
 
+    def add_seqs(self, *seqs):
+        self.prot_nr = len(seqs)
+        self.n = self.prot_nr + sum(len(s) for s in seqs)
+
+        self.buff_shr = sharedctypes.RawArray(ctypes.c_uint8, self.n)
+        self.buff[:] = self.alphabet.translate(
+            np.frombuffer((" ".join(seqs) + " ").encode('ascii'), dtype='|S1')).view(np.uint8)
+
+        self.buff_idx_shr = sharedctypes.RawArray(ctypes.c_uint64, self.prot_nr + 1)
+        for i in range(len(seqs)):
+            self.idx[i + 1] = len(seqs[i]) + 1 + self.idx[i]
+    
+    @lazy_property
+    def buff(self):
+        return np.frombuffer(self.buff_shr, dtype=np.uint8).reshape(self.n)
+    
+    @lazy_property
+    def idx(self):
+        return np.frombuffer(self.buff_idx_shr, dtype=np.uint64).reshape(
+            self.prot_nr + 1)
+
+    def __getitem__(self, i):
+        s = int(self.idx[i])
+        e = int(self.idx[i + 1] - 1)
+        return self.buff[s:e].tobytes().decode("ascii")
+
+
+class SequenceBufferFasta(SequenceBuffer):
+    def __init__(self, fasta_file, alphabet_n=21):
+        seqs, ids = self.parse_fasta(fasta_file)
+        super().__init__(seqs, ids, alphabet_n)
+        
     def parse_fasta(self, fasta_file):
         ids = []
         seqs = []
@@ -386,83 +416,48 @@ class SequenceBuffer(object):
             for rec in SeqIO.parse(fp, "fasta"):
                 ids.append(rec.id)
                 seqs.append(str(rec.seq))
-        self.ids = np.array(ids)
-        return seqs
-
-    def add_seqs(self, *seqs):
-        self.prot_nr = len(seqs)
-        self.n = self.prot_nr + sum(len(s) for s in seqs)
-
-        self.buff_shr = sharedctypes.RawArray(ctypes.c_uint8, self.n)
-        self.buff[:] = np.frombuffer(
-            (" ".join(seqs) + " ").encode("ascii"), dtype=np.uint8
-        )
-
-        self.buff_idx_shr = sharedctypes.RawArray(ctypes.c_uint64, self.prot_nr + 1)
-        for i in range(len(seqs)):
-            self.idx[i + 1] = len(seqs[i]) + 1 + self.idx[i]
-
-    def load_from_db(self):
-        self.prot_nr = len(self._prot_tab)
-        self.n = len(self._seq_buff)
-
-        self.buff_shr = sharedctypes.RawArray(ctypes.c_uint8, self.n)
-        self.buff[:] = self._seq_buff[:].view(np.uint8)
-
-        self.buff_idx_shr = sharedctypes.RawArray(ctypes.c_uint64, self.prot_nr + 1)
-        # self.idx[:-1] = self._prot_tab.cols.SeqOff[:]
-        self.idx[:-1] = self._prot_tab[:]["SeqOff"]
-        self.idx[-1] = self.n
-
-        # store offsets as ids
-        self.ids = np.arange(
-            self.prot_off, self.prot_off + self.prot_nr, dtype=np.uint64
-        )[:, None]
-
-    @lazy_property
-    def buff(self):
-        return np.frombuffer(self.buff_shr, dtype=np.uint8).reshape(self.n)
-
-    @lazy_property
-    def idx(self):
-        return np.frombuffer(self.buff_idx_shr, dtype=np.uint64).reshape(
-            self.prot_nr + 1
-        )
-
-    # @lazy_property
-    # def prot_offsets(self):
-    #     return np.arange(self.prot_off, self.prot_off + self.prot_nr, dtype=np.uint64)
-
-    def __getitem__(self, i):
-        s = int(self.idx[i])
-        e = int(self.idx[i + 1] - 1)
-        return self.buff[s:e].tobytes().decode("ascii")
-
-
-class QuerySequenceBuffer(SequenceBuffer):
-    """
-    get a sliced version of the sequence buffer object for a given species in db
-    TO DO: put an __super__
-    """
-
-    def __init__(self, db, query_sp):
-        self.query_sp = (
-            query_sp if isinstance(query_sp, bytes) else query_sp.encode("ascii")
-        )
-        self.set_query_prot_tab(db)
-        self.filter_query_seq_buff(db)
-        self.load_from_db()
-
-    def set_query_prot_tab(self, db):
-        sp_off = np.searchsorted(db._sp_tab.col("ID"), self.query_sp)
+        return seqs, ids
+    
+class SequenceBufferDB(SequenceBuffer):
+    '''
+    TO DO:
+     - bypass add_seqs
+    '''
+    def __init__(self, prot_offsets, db, alphabet_n=21):
+        seqs, ids = self.load_from_db(prot_offsets, db._prot_tab, db._seq_buff)
+        super().__init__(seqs, list(ids), alphabet_n)
+    
+    @staticmethod
+    def _get_seq(prot_off, prot_tab, seq_buff):
+        prot_ent = prot_tab[prot_off]
+        seq_off = prot_ent['SeqOff']
+        return seq_buff[seq_off:seq_off + prot_ent['SeqLen'] - 1].tobytes().decode("ascii")
+    
+    def load_from_db(self, prot_offsets, prot_tab, seq_buff):
+        seqs = [self._get_seq(prot_off, prot_tab, seq_buff) for prot_off in prot_offsets]
+        return seqs, prot_offsets
+        
+class QuerySequenceBuffer(SequenceBufferDB):
+    '''
+    mimic old QuerySequenceBuffer
+    '''
+    def __init__(self, db, query_sp, alphabet_n=21, fam_filter=np.array([])):
+        self.query_sp = query_sp if isinstance(query_sp, bytes) else query_sp.encode('ascii')
+        prot_offsets = self.get_query_offsets(db, query_sp, fam_filter)
+        super().__init__(prot_offsets, db, alphabet_n)
+        
+    def get_query_offsets(self, db, query_sp, fam_filter):
+        # select all proteins from species
+        sp_off = np.searchsorted(db._sp_tab.col('ID'), self.query_sp)
         sp_ent = db._sp_tab[sp_off]
-        self.prot_off = sp_ent["ProtOff"]
-        self._prot_tab = db._prot_tab[self.prot_off : self.prot_off + sp_ent["ProtNum"]]
+        prot_off = sp_ent['ProtOff']
+        prot_offsets = np.arange(prot_off, prot_off + sp_ent['ProtNum'], dtype=int)
+        # filter queries by family
+        if fam_filter.size > 0:
+            query_fam_offsets = db._prot_tab.col('FamOff')[prot_offsets]
+            return prot_offsets[fam_filter[query_fam_offsets]]
+        else:
+            return prot_offsets
 
-    def filter_query_seq_buff(self, db):
-        self._seq_buff = db._seq_buff[
-            self._prot_tab[0]["SeqOff"] : self._prot_tab[-1]["SeqOff"]
-            + self._prot_tab[-1]["SeqLen"]
-        ]
-        # initialize sequence buffer offset
-        self._prot_tab["SeqOff"] -= db._prot_tab[self.prot_off]["SeqOff"]
+class SequenceBufferOMA(SequenceBuffer):
+    pass
\ No newline at end of file
