@@ -30,15 +30,15 @@ import random
 import tables
 import os
 
-from .hierarchy import get_root_leaf_hog_offsets, get_lca_tax
+from .hierarchy import get_root_leaf_offsets, get_closest_reference_taxon, is_ancestor
 from .merge_search import MergeSearch
 from .index import Index
 
 
 class Validation():
 
-	def __init__(self, db, filename, thresholds, nwk_fn=None, query_sp=None, focal_taxon=None, bin_num=1, val_mode='golike', 
-		neg_root_taxon=None, max_query_nr=None, neg_query_file=None, oma_db_fn=None, nthreads=1):
+	def __init__(self, db, filename, thresholds, oma_db_fn=None, nwk_fn=None, neg_query_file=None, nthreads=1, query_sp=None, max_query_nr=None, 
+		val_mode='golike', neg_root_taxon='random', focal_taxon=None, fam_bin_num=1, hog_bin_num=1, comp_t=0, size_t=0):
 
 		self.db = db
 		self.ki = db.ki
@@ -55,24 +55,27 @@ class Validation():
 			self.va = tables.open_file(self.filename, self.mode)
 			self.va.create_carray('/', 'Thresholds', obj=np.array(thresholds, dtype=np.float64), filters=self.db._compr)
 
-		# required for subfamily validation
-		self.nwk_fn = nwk_fn
-		self.query_sp = query_sp
-		self.focal_taxon = focal_taxon if focal_taxon else self.db.root_taxon
-		self.val_mode = val_mode
-		self.bin_num = bin_num
-
-		# and family validation
-		self.neg_root_taxon = neg_root_taxon
-		self.max_query_nr = max_query_nr
-		self.neg_query_file = neg_query_file
 		self.oma_db_fn = oma_db_fn
+		self.nwk_fn = nwk_fn
+		self.neg_query_file = neg_query_file
 		self.nthreads = nthreads
+		self.query_sp = query_sp
+		self.max_query_nr = max_query_nr
+		self.val_mode = val_mode
+		self.neg_root_taxon = neg_root_taxon
+		self.focal_taxon = focal_taxon if focal_taxon else self.db.root_taxon
+		self.fam_bin_num = fam_bin_num
+		self.hog_bin_num = hog_bin_num
+		self.comp_t = comp_t
+		self.size_t = size_t
 
 		# to pick negative queries
 		random.seed(123)
 		np.random.seed(123)
 
+		# initiate once MergeSearch for negatives
+		self.neg_ms = MergeSearch(ki=self.ki, nthreads=self.nthreads)
+
 	def close(self):
 	    self.va.close()
 
@@ -98,24 +101,34 @@ class Validation():
 	        print("{} already cleaned".format(self.neg_query_file))
 
 	@lazy_property
-	def hog_off2taxbin(self):
+	def fam_off2taxbin(self):
 		tax_off2taxbin = self.bin_taxa(self.nwk_fn, self.focal_taxon, self.db._tax_tab[:], self.query_sp, self.ki.tax_filter, 
-			bin_num=self.bin_num, focal_bin=False, merge_post_lca_taxa=True)
-		return np.array([tax_off2taxbin.get(tax, -1) for tax in self.db._hog_tab.col('TaxOff')], np.int64)
+			bin_num=self.fam_bin_num, focal_bin=False, merge_post_lca_taxa=True)
+		return np.array([tax_off2taxbin.get(tax, -1) for tax in self.db._fam_tab.col('TaxOff')], np.int64)
 
 	@lazy_property
-	def fam_off2taxbin(self):
-		'''
-		one more been because of focal taxon bin
-		'''
+	def hog_off2taxbin(self):
 		tax_off2taxbin = self.bin_taxa(self.nwk_fn, self.focal_taxon, self.db._tax_tab[:], self.query_sp, self.ki.tax_filter, 
-			bin_num=self.bin_num + 1, focal_bin=True, merge_post_lca_taxa=True)
-		return np.array([tax_off2taxbin.get(tax, -1) for tax in self.db._fam_tab.col('TaxOff')], np.int64)
-	
+			bin_num=self.hog_bin_num, focal_bin=False, merge_post_lca_taxa=True)
+		return np.array([tax_off2taxbin.get(tax, -1) for tax in self.db._hog_tab.col('TaxOff')], np.int64)
+
+	@cached_property
+	def fam_filter_lca(self):
+		return self.ki.tax_filter[self.db._hog_tab.col('LCAtaxOff')[self.db._fam_tab.col('HOGoff')]]
+
 	@cached_property
 	def hog_filter_lca(self):
 		return self.ki.tax_filter[self.db._hog_tab.col('LCAtaxOff')]
 
+	@cached_property
+	def fam_filter(self):
+		return ~((self.db._hog_tab.col('CompletenessScore')[self.db._fam_tab.col('HOGoff')] < self.comp_t) + (self.db._hog_tab.col('NrMemberGenes')[self.db._fam_tab.col('HOGoff')] < self.size_t))
+
+	# avoid reloading protein table for each chunk
+	@cached_property
+	def prot_tab(self):
+		return self.db._prot_tab[:]
+
 	### same as in database class; easy access to data ###
 	@property
 	def _thresholds(self):
@@ -135,29 +148,36 @@ class Validation():
 		else:
 			return self.va.get_node('/QueryFamBins')
 
-	def _get_node(self, node):
+	def _get_node_hog(self, node):
 		if '/{}'.format(node) in self.va:
 		    return self.va.get_node('/{}'.format(node))
 		else:
 			# if root-taxon and focal taxon are not the same, there are two additional taxonomic bins for the ancestral and outgroup taxa
-			bn = self.bin_num if self.focal_taxon == self.db.root_taxon else self.bin_num + 2
+			bn = self.hog_bin_num if self.focal_taxon == self.db.root_taxon else self.hog_bin_num + 2
 			return self.va.create_earray('/', node, tables.UInt16Atom(), shape=(0, bn, self._thresholds.nrows), filters=self.db._compr)
 	
 	@property      
 	def _tp_pre(self):
-	    return self._get_node('TP_pre')
+	    return self._get_node_hog('TP_pre')
 
 	@property
 	def _tp_rec(self):
-	    return self._get_node('TP_rec')
+	    return self._get_node_hog('TP_rec')
 	    
 	@property
 	def _fn(self):
-	    return self._get_node('FN')
+	    return self._get_node_hog('FN')
 
 	@property
 	def _fp(self):
-	    return self._get_node('FP')
+	    return self._get_node_hog('FP')
+
+	@property
+	def _pconf(self):
+		if '/PlacementConfig' in self.va:
+			return self.va.root.PlacementConfig
+		else:
+			return self.va.create_earray('/', 'PlacementConfig', tables.UInt8Atom(), shape=(0, self._thresholds.nrows), filters=self.db._compr)
 
 	# for family validation
 	def _get_node_fam(self, node):
@@ -187,22 +207,22 @@ class Validation():
 	    return self._get_node_fam('FP_pos_neg')
 
 	####################################################################################################################################
-	def validate(self, se, prob=False, hog2bin=True, hf_lca=True):
+	def validate(self, se, score, cum_mode, top_m_fams, pvalue_score, perm_nr, w_size, dist, hog2bin=True):
 		'''
-		validate both family and subfamily leves
+		validate both family and subfamily levels
 		'''
-		self.validate_family(se, prob)
-		self.validate_subfamily(se, hog2bin, hf_lca, prob)
+		self.validate_family(se, score, cum_mode, top_m_fams, pvalue_score, perm_nr, w_size, dist)
+		self.validate_subfamily(se, hog2bin, pvalue_score)
 
 	####################################################################################################################################
 	# Subfamily validation
-	def validate_subfamily(self, se, hog2bin=True, hf_lca=True, prob=False):
+	def validate_subfamily(self, se, hog2bin=True, pvalue_score=False):
 		assert (self.mode in {'w', 'a'}), 'Validation must be opened in write mode.'
 
-		tp_pre_query2x2tresh, tp_rec_query2x2tresh, fn_query2x2tresh, fp_query2x2tresh = self._validate_subfamily(
+		tp_pre_query2x2tresh, tp_rec_query2x2tresh, fn_query2x2tresh, fp_query2x2tresh, query2thresh2pconf = self._validate_subfamily(
 			self._thresholds[:], se._queryFam_ranked, se._query_ids, se._queryRankHog_bestpath, se._queryRankHog_scores, 
-			self.db._prot_tab[:], self.db._fam_tab[:], self.db._hog_tab.col('ParentOff'), get_root_leaf_hog_offsets, prob, 
-			self.hog_off2taxbin if hog2bin else np.array([]), self.hog_filter_lca if hf_lca else np.array([]), self.val_mode)
+			self.db._prot_tab[:], self.db._fam_tab[:], self.db._hog_tab.col('ParentOff'), get_root_leaf_offsets, pvalue_score, 
+			self.hog_off2taxbin if hog2bin else np.array([]), self.fam_filter_lca, self.hog_filter_lca, self.val_mode)
 
 		# store results
 		self._tp_pre.append(tp_pre_query2x2tresh)
@@ -220,10 +240,14 @@ class Validation():
 		self._query_ids.flush()
 		self._query_fambins.flush()
 
+		# store also placement configuration (True subfamily, Over-specific, Under-specific, Wrong path, Not predicted)
+		self._pconf.append(query2thresh2pconf)
+		self._pconf.flush()
+
 	@staticmethod
 	def _validate_subfamily(
 		thresholds, queryFam_ranked, query_prot_offsets, queryRankHog_bestpath, queryRankHog_scores,
-		prot_tab, fam_tab, hog2parent, fun_root_leaf, prob, hog2bin, hog_filter_lca, val_mode):
+		prot_tab, fam_tab, hog2parent, fun_root_leaf, pvalue_score, hog2bin, fam_filter_lca, hog_filter_lca, val_mode):
 		'''
 		args:
 		 - pv: whether p-value type of score (the lower the better)
@@ -269,14 +293,46 @@ class Validation():
 			    fn_query2x2tresh[q, fn_x, t_off] = fn_nr
 			    fp_query2x2tresh[q, fp_x, t_off] = fp_nr
 
+		def classify_placement(is_fam_pred, pred_fam, true_fam, true_hogs, pred_hogs, hog2parent):
+			'''
+			Compute placement configuration into:
+			True subfamily, Over-specific, Under-specific, Wrong path, Wrong family, Not predicted
+			'''
+			# wrong path
+			res_type = 3
+			# not predicted
+			if not is_fam_pred:
+				res_type = 5
+			# wrong family
+			elif pred_fam != true_fam:
+				res_type = 4
+			# true subfamily (no true sub-HOG)
+			elif true_hogs.size == 0: 
+			    res_type = 0
+			# under-specific (no predicted sub-HOG)
+			elif pred_hogs.size == 0:
+				res_type = 2
+			# true subfamily 
+			elif pred_hogs[-1] == true_hogs[-1]:
+				res_type = 0
+			# over-specific
+			elif is_ancestor(true_hogs[-1], pred_hogs[-1], hog2parent):
+			    res_type = 1
+			# under-specific
+			elif is_ancestor(pred_hogs[-1], true_hogs[-1], hog2parent):
+			    res_type = 2
+			return res_type
+
 		thresholds = np.array(thresholds, dtype=np.float64)
 
 		bin_num = (np.max(hog2bin) + 1) if hog2bin.size >0 else None
+		
 		# store validation results
 		tp_pre_query2bin2tresh = np.zeros((query_prot_offsets.size, bin_num if bin_num else hog2parent.size, thresholds.size), dtype=np.uint16)
 		tp_rec_query2bin2tresh = np.zeros((query_prot_offsets.size, bin_num if bin_num else hog2parent.size, thresholds.size), dtype=np.uint16)
 		fn_query2bin2tresh = np.zeros((query_prot_offsets.size, bin_num if bin_num else hog2parent.size, thresholds.size), dtype=np.uint16)
 		fp_query2bin2tresh = np.zeros((query_prot_offsets.size, bin_num if bin_num else hog2parent.size, thresholds.size), dtype=np.uint16)
+		query2thresh2pconf = np.zeros((query_prot_offsets.size, thresholds.size), dtype=np.uint8)
 
 		# iterage over queries
 		for q in tqdm(range(query_prot_offsets.size)):
@@ -284,6 +340,11 @@ class Validation():
 			# true data
 			prot_off = query_prot_offsets[q]
 			true_fam = prot_tab[prot_off]['FamOff']
+
+			# skip families specific to a hidden taxon
+			if (fam_filter_lca.size > 0) and fam_filter_lca[true_fam]:
+			    continue
+
 			true_leafhog = prot_tab[prot_off]['HOGoff']
 			true_hogs = fun_root_leaf(true_leafhog, hog2parent)[1:]  # ignore root-HOG
 
@@ -293,7 +354,8 @@ class Validation():
 
 			# pred fam
 			pred_fam = queryFam_ranked[q, 0]
-			    
+			fam_score = queryRankHog_scores[0, q, 0]
+
 			# hogs of pred fam
 			hog_off = fam_tab[pred_fam]['HOGoff']
 			hog_num = fam_tab[pred_fam]['HOGnum']
@@ -309,7 +371,7 @@ class Validation():
 			    t_val = thresholds[t_off]
 			    
 			    # pred hogs
-			    pred_hogs = hogs_bestpath[(hogs_bestpath_score < t_val) if prob else (hogs_bestpath_score >= t_val)]
+			    pred_hogs = hogs_bestpath[(hogs_bestpath_score < t_val) if pvalue_score else (hogs_bestpath_score >= t_val)]
 			    
 			    # confront true classes against predicted classes to get benchmark results
 			    # not supported by numba ...
@@ -320,7 +382,11 @@ class Validation():
 			    _compute_tp_fp_fn(tp_hogs, fn_hogs, fp_hogs, hog2bin, bin_num, tp_pre_query2bin2tresh, tp_rec_query2bin2tresh,
 			        fn_query2bin2tresh, fp_query2bin2tresh, val_mode)
 
-		return tp_pre_query2bin2tresh, tp_rec_query2bin2tresh, fn_query2bin2tresh, fp_query2bin2tresh
+			    # store also placement configuration (True subfamily, Over-specific, Under-specific, Wrong path, Not predicted)
+			    is_fam_pred = (True if fam_score < t_val else False) if pvalue_score else (True if fam_score >= t_val else False)
+			    query2thresh2pconf[q, t_off] = classify_placement(is_fam_pred, pred_fam, true_fam, true_hogs, pred_hogs, hog2parent)
+
+		return tp_pre_query2bin2tresh, tp_rec_query2bin2tresh, fn_query2bin2tresh, fp_query2bin2tresh, query2thresh2pconf
 
 	@staticmethod
 	def bin_taxa(stree_tree, focal_taxon, tax_tab, query_sp, tax_filter, bin_num=2, focal_bin=True, merge_post_lca_taxa=True):
@@ -370,14 +436,14 @@ class Validation():
 	    tax2tax_off = dict(zip(tax_off2tax, range(tax_off2tax.size)))
 	    
 	    # get the lca taxon bewteen the query species and the reference (including hidden taxa with children propagation)
-	    lca_tax = tax_off2tax[get_lca_tax(
+	    lca_tax = tax_off2tax[get_closest_reference_taxon(
 	        tax2tax_off[query_sp.encode('ascii')], tax_tab['ParentOff'], np.argwhere(tax_filter).flatten())].decode('ascii')
 	    
 	    # bin taxa in the species tree
 	    tax2taxbin = _bin_taxa(bin_num, tax2root_dist, lca_tax, focal_bin, merge_post_lca_taxa)
 	    
 	    # get taxa ancestor of focal taxon
-	    older_taxa = tax_off2tax[get_root_leaf_hog_offsets(
+	    older_taxa = tax_off2tax[get_root_leaf_offsets(
 	        tax2tax_off[focal_taxon.encode('ascii')], tax_tab['ParentOff'])][:-1]
 	    
 	    tax_off2taxbin = {}
@@ -413,14 +479,14 @@ class Validation():
 		for thresh in thresholds:
 		    part = np.full(query_values.size, False)
 		    part[(query_values > curr_thresh) & (query_values <= thresh)] = True
-		    partitions.append(part)
+		    partitions.append(np.arange(query_values.size, dtype=np.int64)[part])
 		    part_names.append('{} < {} <= {}'.format(curr_thresh if curr_thresh != -1 else 0, parameter_name, thresh))
 		    curr_thresh = thresh
 
 		# last one
 		part = np.full(query_values.size, False)
 		part[query_values > curr_thresh] = True
-		partitions.append(part)
+		partitions.append(np.arange(query_values.size, dtype=np.int64)[part])
 		part_names.append('{} < {} <= {}'.format(curr_thresh if curr_thresh != -1 else 0, parameter_name, 1))
 
 		return np.array(partitions), part_names
@@ -437,33 +503,29 @@ class Validation():
 
 		part2bin2pre = np.zeros((part_num, bin_num, thresh_num), dtype=np.float64)
 		part2bin2rec = np.zeros((part_num, bin_num, thresh_num), dtype=np.float64)
-		part2bin2tp_pre_nr = np.zeros((part_num, bin_num, thresh_num), dtype=np.uint64)
-		part2bin2tp_rec_nr = np.zeros((part_num, bin_num, thresh_num), dtype=np.uint64)
-		part2bin2fn_nr = np.zeros((part_num, bin_num, thresh_num), dtype=np.uint64)
-		part2bin2fp_nr = np.zeros((part_num, bin_num, thresh_num), dtype=np.uint64)
+		part2bin2true_hog_nr = np.zeros((part_num, bin_num), dtype=np.uint64)
+		part2bin2query_nr = np.zeros((part_num, bin_num), dtype=np.uint64)
 
 		for p in range(part_num):
 		    part = partitions[p]
 		    for b in range(bin_num):
 		        for t in range(thresh_num):
 		            tp_pre_nr = np.sum(tp_pre_query2bin2tresh[:, b, t][part])
-		            tp_rec_nr = np.sum(tp_rec_query2bin2tresh[:, b, t][part])
+		            tp_rec_nr = np.sum(tp_rec_query2bin2tresh[:, b, t][part])  
 		            fn_nr = np.sum(fn_query2bin2tresh[:, b, t][part])
 		            fp_nr = np.sum(fp_query2bin2tresh[:, b, t][part])
 		            part2bin2pre[p, b, t] = (tp_pre_nr/(tp_pre_nr + fp_nr)) if tp_pre_nr or fp_nr else 0
 		            part2bin2rec[p, b, t] = (tp_rec_nr/(tp_rec_nr + fn_nr)) if tp_rec_nr or fn_nr else 0
-		            part2bin2tp_pre_nr[p, b, t] = tp_pre_nr
-		            part2bin2tp_rec_nr[p, b, t] = tp_rec_nr
-		            part2bin2fn_nr[p, b, t] = fn_nr
-		            part2bin2fp_nr[p, b, t] = fp_nr
+		            
+		            # number of true non-hidden subfamily in bin 
+		            part2bin2true_hog_nr[p, b] = np.sum((tp_pre_query2bin2tresh[:, b, t][part] + 
+		                                              fn_query2bin2tresh[:, b, t][part]))
+		            
+		            # number of query from non-hidden subfamilies in bin
+		            part2bin2query_nr[p, b] = np.sum((tp_pre_query2bin2tresh[:, b, t][part] + 
+		                                              fn_query2bin2tresh[:, b, t][part]) != 0)
 
-		part2bin2query_nr = np.zeros((part_num, bin_num), dtype=np.uint64)
-
-		for p in range(part_num):
-		    for b in range(bin_num):
-		        part2bin2query_nr[p, b] = part2bin2tp_rec_nr[p, b, 0] + part2bin2fn_nr[p, b, 0]
-		        
-		return part2bin2pre, part2bin2rec, part2bin2query_nr
+		return part2bin2pre, part2bin2rec, part2bin2true_hog_nr, part2bin2query_nr
 
 	def compute_precision_recall_subfamily(self, partitions=np.array([])):
 		return self._compute_precision_recall_subfamily(
@@ -487,7 +549,7 @@ class Validation():
 		return part2bin2f1_max, part2bin2f1_tval, part2bin2f1_toff
 
 	def F1max_subfamily(self, partitions=np.array([])):
-		part2bin2pre, part2bin2rec, part2bin2query_nr = self.compute_precision_recall_subfamily(partitions)
+		part2bin2pre, part2bin2rec, part2bin2true_hog_nr, part2bin2query_nr = self.compute_precision_recall_subfamily(partitions)
 		return self._F1max_subfamily(part2bin2pre, part2bin2rec, self._thresholds[:])
 
 	@staticmethod
@@ -511,19 +573,19 @@ class Validation():
 		return part2bin2f1_max, part2bin2f1_tval, part2bin2f1_toff
 
 	def PREpro_subfamily(self, partitions=np.array([])):
-		part2bin2pre, part2bin2rec, part2bin2query_nr = self.compute_precision_recall(partitions)
+		part2bin2pre, part2bin2rec, part2bin2true_hog_nr, part2bin2query_nr = self.compute_precision_recall(partitions)
 		return self._PREpro_subfamily(part2bin2pre, part2bin2rec, self._thresholds[:])
 
 	####################################################################################################################################
 	# Family validation
 	@lazy_property
 	def clade_specific_negatives(self):
-		return self.get_clade_specific_negatives(self.nwk_fn, self.oma_db_fn, self.neg_root_taxon, self.db.min_prot_nr, self.max_query_nr)
+		return self.get_clade_specific_negatives(self.nwk_fn, self.oma_db_fn, self.neg_root_taxon, self.db.min_fam_size, self.max_query_nr)
 
-	def validate_family(self, se, prob=False):
+	def validate_family(self, se, score, cum_mode, top_m_fams, pvalue_score, perm_nr, w_size, dist):
 
 		# get the negative queries
-		if self.neg_root_taxon:
+		if self.neg_root_taxon != 'random':
 			all_neg_seqs, all_neg_ids = self.clade_specific_negatives
 			query_i = self._query_ids.nrows
 			query_j = query_i + se._query_ids.size
@@ -531,7 +593,7 @@ class Validation():
 			neg_seqs = all_neg_seqs[query_i: query_j]
 			neg_ids = all_neg_ids[query_i: query_j]
 		else:
-			neg_seqs, neg_ids = self.get_random_negatives(self.db._prot_tab.col('SeqLen')[se._query_ids[:]])
+			neg_seqs, neg_ids = self.get_random_negatives(self.prot_tab['SeqLen'][se._query_ids[:]])
 
 		# store negatives in fasta file
 		with open(self.neg_query_file, 'a') as ff:
@@ -539,14 +601,17 @@ class Validation():
 		        ff.write(">{}\n{}\n".format(neg_ids[i], s))
 
 		# search negatives
-		neg_ms = MergeSearch(ki=self.ki, nthreads=self.nthreads)
-		neg_ms.merge_search(seqs=neg_seqs, ids=neg_ids)
+		# neg_ms = MergeSearch(ki=self.ki, nthreads=self.nthreads)
+		self.neg_ms.merge_search(seqs=neg_seqs, ids=neg_ids, score=score, cum_mode=cum_mode, top_m_fams=top_m_fams, perm_nr=perm_nr, w_size=w_size, dist=dist,
+			fam_filter=self.fam_filter)
 
 		# validate negatives
-		tn_query2tresh, fp_neg_query2tresh = self._validate_negative(self._thresholds[:], neg_ms._queryFam_ranked, neg_ms._queryFam_scores, prob)
+		tn_query2tresh, fp_neg_query2tresh = self._validate_negative(
+			self._thresholds[:], self.neg_ms._queryFam_ranked, self.neg_ms._queryFam_scores, pvalue_score)
 
 		# validate positives
-		tp_query2tresh, fn_query2tresh, fp_pos_query2tresh = self._validate_positive(se._query_ids, self._thresholds[:], se._queryFam_ranked, se._queryFam_scores, self.db._prot_tab[:], prob)
+		tp_query2tresh, fn_query2tresh, fp_pos_query2tresh = self._validate_positive(
+			se._query_ids, self._thresholds[:], se._queryFam_ranked, se._queryFam_scores, self.db._prot_tab[:], pvalue_score, self.fam_filter_lca)
 
 		# store results
 		self._fam_tn.append(tn_query2tresh)
@@ -561,7 +626,7 @@ class Validation():
 		self._fam_fp_pos.flush()
 
 	@staticmethod
-	def _validate_positive(query_prot_offsets, thresholds, queryFam_ranked, queryFam_scores, prot_tab, prob):
+	def _validate_positive(query_prot_offsets, thresholds, queryFam_ranked, queryFam_scores, prot_tab, pvalue_score, fam_filter_lca):
 	    '''
 	    results from the positive query set
 	    '''
@@ -579,6 +644,10 @@ class Validation():
 	        prot_off = query_prot_offsets[q]
 	        true_fam = prot_tab[prot_off]['FamOff']
 
+	        # skip families specific to a hidden taxon
+	        if (fam_filter_lca.size > 0) and fam_filter_lca[true_fam]:
+	            continue
+
 	        # pred data
 	        pred_fam = queryFam_ranked[q]
 	        score = queryFam_scores[q]
@@ -587,23 +656,27 @@ class Validation():
 	        for t_off in range(thresholds.size):
 	            t_val = thresholds[t_off]
 
-	            is_pred = (True if score < t_val else False) if prob else (True if score >= t_val else False)
+	            is_pred = (True if score < t_val else False) if pvalue_score else (True if score >= t_val else False)
 
-	            # TP
 	            if is_pred:
+	                
+	                # TP
 	                if pred_fam==true_fam:
 	                    tp_query2tresh[q, t_off] = True
-	                # FP
+	                
+	                # FP and FN
 	                else:
 	                    fp_query2tresh[q, t_off] = True
-	            # FN
+	                    fn_query2tresh[q, t_off] = True
+	            
+	            # FN only
 	            else:
 	                fn_query2tresh[q, t_off] = True
 
 	    return tp_query2tresh, fn_query2tresh, fp_query2tresh
 
 	@staticmethod
-	def _validate_negative(thresholds, queryFam_ranked, queryFam_scores, prob):
+	def _validate_negative(thresholds, queryFam_ranked, queryFam_scores, pvalue_score):
 	    '''
 	    results from the negative query set
 	    '''
@@ -624,7 +697,7 @@ class Validation():
 	        for t_off in range(thresholds.size):
 	            t_val = thresholds[t_off]
 
-	            is_pred = (True if score < t_val else False) if prob else (True if score >= t_val else False)
+	            is_pred = (True if score < t_val else False) if pvalue_score else (True if score >= t_val else False)
 
 	            # FP
 	            if is_pred:
@@ -655,8 +728,7 @@ class Validation():
 
 	    return np.array(sorted(taxa)), species
 
-	@staticmethod
-	def get_clade_specific_negatives(stree_path, oma_h5_path, neg_root_taxon, min_prot_nr, max_query_nr):
+	def get_clade_specific_negatives(self, stree_path, oma_h5_path, neg_root_taxon, min_fam_size, max_query_nr):
 	    '''
 	    1.	precompute clade taxonomic levels
 	    2.	iterate over hog tab and store families with all taxa within clade of interest and with >6 members
@@ -738,7 +810,7 @@ class Validation():
 	    for fam, ent_offsets in tqdm(fam2ent_offsets.items()):
 
 	        # same filter than positive queries
-	        if len(ent_offsets) >= min_prot_nr:
+	        if len(ent_offsets) >= min_fam_size:
 
 	            fam2ent_off[fam] = random.choice(ent_offsets)
 
@@ -814,10 +886,11 @@ class Validation():
 		        neg_part2fp_nr[p, t] = neg_fp_nr
 		        part2fp_nr[p, t] = fp_nr
 
-		# compute number of queries
+		# query number (union of positive fp, fn and tp)
 		part2query_nr = np.zeros((part_num), dtype=np.uint64)
 		for p in range(part_num):
-		    part2query_nr[p] = part2tp_nr[p, 0] + part2fn_nr[p, 0] + pos_part2fp_nr[p, 0]
+		    part = partitions[p]
+		    part2query_nr[p] = np.sum(np.array(fp_pos_query2tresh[:, 0][part], dtype=np.bool) + np.array(fn_query2tresh[:, 0][part], dtype=np.bool) + np.array(tp_query2tresh[:, 0][part], dtype=np.bool))
 
 		return part2pre, part2rec, part2spe, part2query_nr
 
